#include "stm32f10x.h"
#include "board.h"

void I2C_Configuration(void)
{
	I2C_InitTypeDef I2C_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE); 

  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  	GPIO_Init(GPIOB, &GPIO_InitStructure); 

  	I2C_Cmd(I2C1, DISABLE);
  	I2C_DeInit(I2C1);

	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_OwnAddress1 = 0x99;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = 100000;

	I2C_Cmd(I2C1, ENABLE);

	I2C_Init(I2C1, &I2C_InitStructure); 
}

unsigned char I2C_READ_BUFFER_FROM_DEVICE(unsigned char  DEVICE_ADDRESS,
										  unsigned int   INTERNAL_ADDRESS,
										  unsigned char* RX_BUFFER,
									      unsigned int   number_of_bytes)
{
  unsigned int licznik = 0;

  while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));

  I2C_GenerateSTART(I2C1, ENABLE);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));


  I2C_Send7bitAddress(I2C1, DEVICE_ADDRESS, I2C_Direction_Transmitter);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

  I2C_Cmd(I2C1, ENABLE);

  if(INTERNAL_ADDRESS & 0xFF00)
  {
	  I2C_SendData(I2C1, ((INTERNAL_ADDRESS>>8)&0x00FF));
	  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  }

  I2C_SendData(I2C1, INTERNAL_ADDRESS&0x00FF);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  I2C_GenerateSTART(I2C1, ENABLE);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

  I2C_Send7bitAddress(I2C1, DEVICE_ADDRESS, I2C_Direction_Receiver);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));


  for(licznik=1; licznik<number_of_bytes; licznik++)
  {
	  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));

	  if((licznik+1)==number_of_bytes)
	  {
		  I2C_AcknowledgeConfig(I2C1, DISABLE);
	  }
	  *(RX_BUFFER++) = I2C_ReceiveData(I2C1);
  }

  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
  *(RX_BUFFER) = I2C_ReceiveData(I2C1);
  I2C_GenerateSTOP(I2C1, ENABLE);
  I2C_AcknowledgeConfig(I2C1, ENABLE);
  return 0;
}

unsigned char I2C_WRITE_BUFFER_TO_DEVICE(unsigned char  DEVICE_ADDRESS,
										 unsigned int   INTERNAL_ADDRESS,
										 unsigned char* TX_BUFFER,
										 unsigned int   number_of_bytes)
{
  unsigned int licznik = 0;

  while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));

  I2C_GenerateSTART(I2C1, ENABLE);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

  I2C_Send7bitAddress(I2C1, DEVICE_ADDRESS, I2C_Direction_Transmitter);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

  I2C_Cmd(I2C1, ENABLE);

  if(INTERNAL_ADDRESS & 0xFF00)
  {
	  I2C_SendData(I2C1, ((INTERNAL_ADDRESS>>8)&0x00FF));
	  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  }

  I2C_SendData(I2C1, INTERNAL_ADDRESS&0x00FF);
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  for(licznik=0; licznik<number_of_bytes; licznik++)
  {
	  I2C_SendData(I2C1, *(TX_BUFFER++));
	  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  }

  I2C_GenerateSTOP(I2C1, ENABLE);

  return 0;
}
